@@ -161,13 +161,13 @@ new_snapped_position (GstClockTime distance)
 }
 
 static GHashTable *
-new_edit_table ()
+new_edit_table (void)
 {
   return g_hash_table_new_full (NULL, NULL, NULL, g_free);
 }
 
 static GHashTable *
-new_position_table ()
+new_position_table (void)
 {
   return g_hash_table_new_full (NULL, NULL, NULL, g_free);
 }
@@ -2484,7 +2484,7 @@ create_transitions (GNode * node,
   if (!ges_layer_get_auto_transition (layer))
     return FALSE;
 
-  GST_LOG (node->data, "Checking for overlaps");
+  GST_LOG_OBJECT (node->data, "Checking for overlaps");
   data.root = g_node_get_root (node);
   check_all_overlaps_with_element (node, &data);
 
